"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.FfmpegProcess = void 0;
const dgram_1 = require("dgram");
const execa_1 = require("execa");
class FfmpegProcess {
    constructor(delegate, sessionId, command, returnPort, callback) {
        this.command = command.join(' ');
        this.delegate = delegate;
        this.log = delegate.log;
        this.name = delegate.cameraName;
        this.sessionId = sessionId;
        this.log.debug('%s: ffmpeg command: %s %s', this.name, delegate.videoProcessor, command.join(' '));
        if (returnPort) {
            this.createSocket(returnPort);
        }
        void this.startFfmpeg(command, callback);
    }
    createSocket(portInfo) {
        const socket = (0, dgram_1.createSocket)(portInfo.addressVersion === 'ipv6' ? 'udp6' : 'udp4');
        socket.on('error', (error) => {
            this.log.error('%s: Socket error: %s.', this.name, error.name);
            this.delegate.stopStream(this.sessionId);
        });
        socket.on('message', () => {
            if (this.streamTimeout) {
                clearTimeout(this.streamTimeout);
            }
            this.streamTimeout = setTimeout(() => {
                this.log.debug('%s: video stream appears to be inactive for 5 seconds. Stopping stream.', this.name);
                this.delegate.controller.forceStopStreamingSession(this.sessionId);
                this.delegate.stopStream(this.sessionId);
            }, 5000);
        });
        socket.bind(portInfo.port);
    }
    async startFfmpeg(ffmpegCommandLine, callback) {
        var _a, _b, _c, _d;
        let started = false;
        this.process = (0, execa_1.execa)(this.delegate.videoProcessor, ffmpegCommandLine);
        (_a = this.process.stdin) === null || _a === void 0 ? void 0 : _a.on('error', (error) => {
            if (!error.message.includes('EPIPE')) {
                this.log.error('%s: FFmpeg error: %s.', this.name, error.message);
            }
        });
        (_b = this.process.stderr) === null || _b === void 0 ? void 0 : _b.on('data', (data) => {
            if (!started) {
                started = true;
                this.log.debug('%s: Received the first frame.', this.name);
                if (callback) {
                    callback();
                }
            }
        });
        try {
            await this.process;
        }
        catch (error) {
            if (!(error instanceof Error)) {
                this.log.error('Unknown error received while attempting to start FFmpeg: %s.', error);
                return;
            }
            const execError = error;
            const logPrefix = this.name + ': FFmpeg process ended ';
            const code = (_c = execError.exitCode) !== null && _c !== void 0 ? _c : null;
            const signal = (_d = execError.signal) !== null && _d !== void 0 ? _d : null;
            if (execError.isCanceled) {
                this.log.debug(logPrefix + '(Expected).');
                return;
            }
            const errorMessage = logPrefix + '(Error).' + (code === null ? '' : ' Exit code: ' + code.toString() + '.') + (signal === null ? '' : ' Signal: ' + signal + '.');
            this.log.error('%s: %s', this.name, execError.message);
            this.delegate.stopStream(this.sessionId);
            if (!started && callback) {
                callback(new Error(errorMessage));
                return;
            }
            this.delegate.controller.forceStopStreamingSession(this.sessionId);
        }
    }
    stop() {
        this.process.cancel();
    }
    getStdin() {
        return this.process.stdin;
    }
    getStdout() {
        return this.process.stdout;
    }
    static async codecEnabled(videoProcessor, codec) {
        const output = await (0, execa_1.execa)(videoProcessor, ['-codecs']);
        return output.stdout.includes(codec);
    }
}
exports.FfmpegProcess = FfmpegProcess;
//# sourceMappingURL=ffmpeg-process.js.map