"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.UnifiStreamingDelegate = void 0;
const image_utils_1 = require("../utils/image-utils");
const unifi_1 = require("../unifi/unifi");
const ffmpeg_process_1 = require("./ffmpeg-process");
const rtp_splitter_1 = require("./rtp-splitter");
const ffmpeg_for_homebridge_1 = __importDefault(require("ffmpeg-for-homebridge"));
class UnifiStreamingDelegate {
    constructor(camera, log, api, cameraConfig, videoProcessor) {
        this.camera = camera;
        this.cameraName = camera.name;
        this.api = api;
        this.hap = api.hap;
        this.log = log;
        this.ongoingSessions = {};
        this.pendingSessions = {};
        this.videoProcessor = videoProcessor || ffmpeg_for_homebridge_1.default || 'ffmpeg';
        log.info('VIDEO PROCESSOR: ' + this.videoProcessor);
        this.cameraConfig = cameraConfig;
        const options = {
            cameraStreamCount: 10,
            delegate: this,
            streamingOptions: {
                audio: {
                    codecs: [
                        {
                            samplerate: 16,
                            type: "AAC-eld"
                        }
                    ],
                    twoWayAudio: this.camera.supportsTwoWayAudio
                },
                supportedCryptoSuites: [0],
                video: {
                    codec: {
                        levels: [0, 1, 2],
                        profiles: [1]
                    },
                    resolutions: [
                        [1920, 1080, 30],
                        [1280, 960, 30],
                        [1280, 720, 30],
                        [1024, 768, 30],
                        [640, 480, 30],
                        [640, 360, 30],
                        [480, 360, 30],
                        [480, 270, 30],
                        [320, 240, 30],
                        [320, 240, 15],
                        [320, 180, 30]
                    ]
                }
            }
        };
        this.controller = new this.hap.CameraController(options);
    }
    async handleSnapshotRequest(request, callback) {
        this.log.debug('Handling snapshot request for Camera: ' + this.camera.name);
        if (!this.camera || !this.camera.lastDetectionSnapshot) {
            this.log.debug('Getting new snapshot');
            try {
                const snapshotData = await UnifiStreamingDelegate.uFlows.getCameraSnapshot(this.camera, request.width, request.height);
                callback(undefined, snapshotData);
            }
            catch (error) {
                callback(undefined, null);
            }
        }
        else {
            this.log.debug('Returning annotated snapshot');
            const canvas = image_utils_1.ImageUtils.resizeCanvas(this.camera.lastDetectionSnapshot, request.width, request.height);
            callback(undefined, canvas.toBuffer('image/jpeg'));
        }
    }
    async handleStreamRequest(request, callback) {
        switch (request.type) {
            case "start":
                this.startStream(request, callback);
                break;
            case "reconfigure":
                this.log.debug('%s: Ignoring request to reconfigure: %sx%s, %s fps, %s kbps.', this.camera.name, request.video.width, request.video.height, request.video.fps, request.video.max_bit_rate);
                callback();
                break;
            case "stop":
            default:
                this.stopStream(request.sessionID);
                callback();
                break;
        }
    }
    startStream(request, callback) {
        var _a, _b;
        const sessionInfo = this.pendingSessions[request.sessionID];
        const sdpIpVersion = sessionInfo.addressVersion === 'ipv6' ? 'IP6 ' : 'IP4';
        const videomtu = 188 * 3;
        const audiomtu = 188;
        const streamingUrl = unifi_1.Unifi.generateStreamingUrlForBestMatchingResolution(this.cameraConfig.source, this.camera.streams, request.video.width, request.video.height);
        const ffmpegArgs = ['-hide_banner', '-rtsp_transport', 'tcp', '-i', streamingUrl];
        this.log.info('%s: HomeKit video stream request received: %sx%s, %s fps, %s kbps.', this.camera.name, request.video.width, request.video.height, request.video.fps, request.video.max_bit_rate);
        this.log.info('%s: Selected stream: %s for playback', this.camera.name, streamingUrl);
        ffmpegArgs.push('-map', '0:v', '-vcodec', 'copy', '-f', 'rawvideo', '-pix_fmt', 'yuvj420p', '-r', request.video.fps.toString(), '-b:v', request.video.max_bit_rate.toString() + 'k', '-bufsize', (2 * request.video.max_bit_rate).toString() + 'k', '-maxrate', request.video.max_bit_rate.toString() + 'k', '-payload_type', request.video.pt.toString(), '-ssrc', sessionInfo.videoSSRC.toString(), '-f', 'rtp', '-srtp_out_suite', 'AES_CM_128_HMAC_SHA1_80', '-srtp_out_params', sessionInfo.videoSRTP.toString('base64'), 'srtp://' + sessionInfo.address + ':' + sessionInfo.videoPort.toString() + '?rtcpport=' + sessionInfo.videoPort.toString() +
            '&localrtcpport=' + sessionInfo.videoPort.toString() + '&pkt_size=' + videomtu.toString());
        if (sessionInfo.hasLibFdk) {
            ffmpegArgs.push('-map', '0:a', '-acodec', 'libfdk_aac', '-profile:a', 'aac_eld', '-flags', '+global_header', '-f', 'null', '-ar', request.audio.sample_rate.toString() + 'k', '-b:a', request.audio.max_bit_rate.toString() + 'k', '-bufsize', (2 * request.audio.max_bit_rate).toString() + 'k', '-ac', '1');
            ffmpegArgs.push('-payload_type', request.audio.pt.toString(), '-ssrc', sessionInfo.audioSSRC.toString(), '-f', 'rtp', '-srtp_out_suite', 'AES_CM_128_HMAC_SHA1_80', '-srtp_out_params', sessionInfo.audioSRTP.toString('base64'), 'srtp://' + sessionInfo.address + ':' + sessionInfo.audioPort.toString() + '?rtcpport=' + sessionInfo.audioPort.toString() +
                '&localrtcpport=' + sessionInfo.audioPort.toString() + '&pkt_size=' + audiomtu.toString());
        }
        const ffmpeg = new ffmpeg_process_1.FfmpegProcess(this, request.sessionID, ffmpegArgs, (sessionInfo.hasLibFdk && this.camera.supportsTwoWayAudio) ? undefined : {
            addressVersion: sessionInfo.addressVersion,
            port: sessionInfo.videoReturnPort
        }, callback);
        this.ongoingSessions[request.sessionID] = { ffmpeg: [ffmpeg], rtpDemuxer: sessionInfo.rtpDemuxer };
        delete this.pendingSessions[request.sessionID];
        if (!sessionInfo.hasLibFdk || !this.camera.supportsTwoWayAudio) {
            return;
        }
        const sdpReturnAudio = [
            'v=0',
            'o=- 0 0 IN ' + sdpIpVersion + ' 127.0.0.1',
            's=' + this.camera.name + ' Audio Talkback',
            'c=IN ' + sdpIpVersion + ' ' + sessionInfo.address,
            't=0 0',
            'm=audio ' + sessionInfo.audioIncomingRtpPort.toString() + ' RTP/AVP 110',
            'b=AS:24',
            'a=rtpmap:110 MPEG4-GENERIC/16000/1',
            'a=fmtp:110 profile-level-id=1;mode=AAC-hbr;sizelength=13;indexlength=3;indexdeltalength=3; config=F8F0212C00BC00',
            'a=crypto:1 AES_CM_128_HMAC_SHA1_80 inline:' + sessionInfo.audioSRTP.toString('base64')
        ].join('\n');
        const ffmpegReturnAudioCmd = [
            '-hide_banner',
            '-protocol_whitelist', 'pipe,udp,rtp,file,crypto',
            '-f', 'sdp',
            '-acodec', 'libfdk_aac',
            '-i', 'pipe:0',
            '-acodec', this.camera.talkbackSettings.typeFmt,
            '-flags', '+global_header',
            '-ar', this.camera.talkbackSettings.samplingRate.toString(),
            '-b:a', '64k',
            '-ac', this.camera.talkbackSettings.channels.toString(),
            '-f', 'adts',
            'udp://' + this.camera.ip + ':' + this.camera.talkbackSettings.bindPort.toString()
        ];
        const ffmpegReturnAudio = new ffmpeg_process_1.FfmpegProcess(this, request.sessionID, ffmpegReturnAudioCmd);
        this.ongoingSessions[request.sessionID].ffmpeg.push(ffmpegReturnAudio);
        (_a = ffmpegReturnAudio.getStdin()) === null || _a === void 0 ? void 0 : _a.write(sdpReturnAudio);
        (_b = ffmpegReturnAudio.getStdin()) === null || _b === void 0 ? void 0 : _b.end();
    }
    async prepareStream(request, callback) {
        const hasLibFdk = await ffmpeg_process_1.FfmpegProcess.codecEnabled(this.videoProcessor, 'libfdk_aac');
        const audioIncomingRtcpPort = (await rtp_splitter_1.RtpUtils.reservePorts())[0];
        const audioIncomingPort = (hasLibFdk && this.camera.supportsTwoWayAudio) ? (await rtp_splitter_1.RtpUtils.reservePorts())[0] : -1;
        const audioIncomingRtpPort = (hasLibFdk && this.camera.supportsTwoWayAudio) ? (await rtp_splitter_1.RtpUtils.reservePorts(2))[0] : -1;
        const audioSSRC = this.hap.CameraController.generateSynchronisationSource();
        if (!hasLibFdk) {
            this.log.info('%s: Audio support disabled. A version of FFmpeg that is compiled with fdk_aac support is required to support audio.', this.camera.name);
        }
        const rtpDemuxer = (hasLibFdk && this.camera.supportsTwoWayAudio) ?
            new rtp_splitter_1.RtpDemuxer(this, request.addressVersion, audioIncomingPort, audioIncomingRtcpPort, audioIncomingRtpPort) : null;
        const videoReturnPort = (await rtp_splitter_1.RtpUtils.reservePorts())[0];
        const videoSSRC = this.hap.CameraController.generateSynchronisationSource();
        const sessionInfo = {
            address: request.targetAddress,
            addressVersion: request.addressVersion,
            audioCryptoSuite: request.audio.srtpCryptoSuite,
            audioIncomingRtcpPort: audioIncomingRtcpPort,
            audioIncomingRtpPort: audioIncomingRtpPort,
            audioPort: request.audio.port,
            audioSRTP: Buffer.concat([request.audio.srtp_key, request.audio.srtp_salt]),
            audioSSRC: audioSSRC,
            hasLibFdk: hasLibFdk,
            rtpDemuxer: rtpDemuxer,
            videoCryptoSuite: request.video.srtpCryptoSuite,
            videoPort: request.video.port,
            videoReturnPort: videoReturnPort,
            videoSRTP: Buffer.concat([request.video.srtp_key, request.video.srtp_salt]),
            videoSSRC: videoSSRC
        };
        const response = {
            audio: {
                port: (hasLibFdk && this.camera.supportsTwoWayAudio) ? audioIncomingPort : audioIncomingRtcpPort,
                srtp_key: request.audio.srtp_key,
                srtp_salt: request.audio.srtp_salt,
                ssrc: audioSSRC
            },
            video: {
                port: videoReturnPort,
                srtp_key: request.video.srtp_key,
                srtp_salt: request.video.srtp_salt,
                ssrc: videoSSRC
            }
        };
        this.pendingSessions[request.sessionID] = sessionInfo;
        callback(undefined, response);
    }
    stopStream(sessionId) {
        var _a, _b;
        try {
            if (this.ongoingSessions[sessionId]) {
                for (const ffmpegProcess of this.ongoingSessions[sessionId].ffmpeg) {
                    ffmpegProcess.stop();
                }
            }
            (_b = (_a = this.ongoingSessions[sessionId]) === null || _a === void 0 ? void 0 : _a.rtpDemuxer) === null || _b === void 0 ? void 0 : _b.close();
            delete this.pendingSessions[sessionId];
            delete this.ongoingSessions[sessionId];
            this.log.info('%s: Stopped video streaming session.', this.camera.name);
        }
        catch (error) {
            this.log.error('%s: Error occurred while ending the FFmpeg video processes: %s.', this.camera.name, error);
        }
    }
    shutdown() {
        for (const session of Object.keys(this.ongoingSessions)) {
            this.stopStream(session);
        }
    }
}
exports.UnifiStreamingDelegate = UnifiStreamingDelegate;
//# sourceMappingURL=unifi-streaming-delegate.js.map