"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.GooglePhotos = void 0;
const googleapis_1 = require("googleapis");
const http = require('http');
const fs = require('fs');
const { promisify } = require('util');
const Photos = require('googlephotos');
const readFileAsync = promisify(fs.readFile);
const writeFileAsync = promisify(fs.writeFile);
class GooglePhotos {
    constructor(config, userStoragePath, log) {
        this.initPerformed = false;
        this.config = config;
        this.userStoragePath = userStoragePath;
        this.log = log;
        setTimeout(async () => {
            await this.init();
        });
    }
    async uploadImage(imagePath, imageName, description) {
        if (!this.initPerformed) {
            this.log.debug('Google Photos logic could not start or is still starting...');
            return null;
        }
        try {
            const photos = new Photos(await this.authenticate());
            const response = await photos.mediaItems.upload(this.gPhotosPersistData.albumId, imageName, imagePath, description);
            return response.newMediaItemResults[0].mediaItem.productUrl;
        }
        catch (error) {
            this.log.debug('Uploading to Google Photos failed');
            this.log.debug(error);
            throw new Error('Cannot upload image to Google Photos!');
        }
    }
    async init() {
        try {
            this.gPhotosPersistData = await this.readConfig();
        }
        catch (error) {
            this.gPhotosPersistData = {
                auth_refresh_token: null,
                albumId: null
            };
            this.log.debug('Google photos persisted data cannot be read/parsed, initial setup!');
            this.log.debug(error);
        }
        this.log.debug(JSON.stringify(this.gPhotosPersistData, null, 4));
        if (!this.config || !this.config.auth_clientId || !this.config.auth_clientSecret || !this.config.auth_redirectUrl) {
            this.log.debug('Google photos config not correct/incomplete! Disabling functionality!');
            return;
        }
        this.oauth2Client = new googleapis_1.google.auth.OAuth2(this.config.auth_clientId, this.config.auth_clientSecret, this.config.auth_redirectUrl);
        const photos = new Photos(await this.authenticate());
        try {
            if (!this.gPhotosPersistData.albumId) {
                this.log.debug('Creating Google Photos album');
                const response = await photos.albums.create('Homebridge-Unifi-Protect-Motion-Captures');
                this.gPhotosPersistData.albumId = response.id;
            }
            else {
                this.log.debug('Google Photos album already created');
            }
            await this.writeConfig(this.gPhotosPersistData);
            this.initPerformed = true;
        }
        catch (error) {
            this.log.debug('Could not create album');
            this.log.debug(error);
        }
    }
    async authenticate() {
        if (!this.gPhotosPersistData.auth_refresh_token) {
            const url = this.oauth2Client.generateAuthUrl({
                access_type: 'offline',
                scope: [Photos.Scopes.READ_AND_APPEND]
            });
            this.log.info('Please log in on Google Photos to allow for uploading: ' + url);
        }
        else {
            this.oauth2Client.setCredentials({
                refresh_token: this.gPhotosPersistData.auth_refresh_token
            });
        }
        let accessToken = null;
        try {
            if (this.gPhotosPersistData.auth_refresh_token) {
                this.log.debug('Refreshing access token');
                accessToken = (await this.oauth2Client.getAccessToken()).token;
            }
            else {
                this.log.debug('Fetching new access token');
                const { tokens } = await this.oauth2Client.getToken(await this.getAuthCodeFromOauth2callback());
                this.oauth2Client.setCredentials(tokens);
                if (tokens.refresh_token) {
                    this.gPhotosPersistData.auth_refresh_token = tokens.refresh_token;
                }
                this.log.debug(JSON.stringify(tokens, null, 4));
                accessToken = tokens.access_token;
            }
            return accessToken;
        }
        catch (error) {
            this.log.debug('Failed to get auth!');
            this.log.debug(error);
        }
    }
    getAuthCodeFromOauth2callback() {
        return new Promise((resolve, reject) => {
            const requestHandler = (request, response) => {
                response.statusCode = 200;
                response.setHeader('Content-Type', 'text/plain');
                const url = new URL('http://localhost' + request.url);
                if (url.pathname === '/oauth2-callback') {
                    response.end('OAuth2 callback handled!');
                    server.close();
                    resolve(url.searchParams.get('code'));
                }
                else {
                    response.end('OAuth2 callback handler running...');
                }
            };
            const server = http.createServer(requestHandler).listen(8888);
        });
    }
    async readConfig() {
        return JSON.parse(await readFileAsync(this.userStoragePath + '/unifi-protect-google-settings.json'));
    }
    async writeConfig(config) {
        return writeFileAsync(this.userStoragePath + '/unifi-protect-google-settings.json', JSON.stringify(config, null, 4));
    }
}
exports.GooglePhotos = GooglePhotos;
//# sourceMappingURL=google-photos.js.map